//
// LSMScheduler
//
#include <windows.h>
#include "LSMScheduler.h"
#pragma comment(lib,"lsmodule.lib")

// The mandatory pre-setup function.  Our name is "LSMScheduler", and the class is LSMScheduler.
// This sets up a "ModulePath" variable which contains the path to this module in case we want it,
// and a "PluginLog" variable, which contains the path and filename of what we should use for our
// debug logging if we need it.  It also sets up a variable "pModule" which is the pointer to
// our instanced class.
LSModulePreSetup("Scheduler",LSMScheduler);

// Basic LavishScript datatypes, these get retrieved on startup by our initialize function, so we can
// use them in our Top-Level Objects or custom datatypes
LSType *pStringType=0;
LSType *pIntType=0;
LSType *pBoolType=0;
LSType *pFloatType=0;
LSType *pTimeType=0;
LSType *pByteType=0;
LSType *pIntPtrType=0;
LSType *pBoolPtrType=0;
LSType *pFloatPtrType=0;
LSType *pBytePtrType=0;
#define DATATYPE(_class_,_variable_) class _class_ *_variable_=0;
#include "DataTypes.h"
#undef DATATYPE

LSInterface *pLSInterface=0;

// The constructor of our class.  General initialization cannot be done yet, because we're not given
// the pointer to the Inner Space interface until it is ready for us to initialize.  Just set the
// pointer we have to the interface to 0.  Initialize data members, too.
LSMScheduler::LSMScheduler(void)
{
}

// Free any remaining resources in the destructor.  This is called when the DLL is unloaded, but
// Inner Space calls the "Shutdown" function first.  Most if not all of the shutdown process should
// be done in Shutdown.
LSMScheduler::~LSMScheduler(void)
{
}

// Initialize is called by Inner Space when the extension should initialize.
bool LSMScheduler::Initialize(LSInterface *p_LSInterface)
{
	pLSInterface=p_LSInterface;

	// retrieve basic ISData types
	pStringType=pLSInterface->FindLSType("string");
	pIntType=pLSInterface->FindLSType("int");
	pBoolType=pLSInterface->FindLSType("bool");
	pFloatType=pLSInterface->FindLSType("float");
	pTimeType=pLSInterface->FindLSType("time");
	pByteType=pLSInterface->FindLSType("byte");
	pIntPtrType=pLSInterface->FindLSType("intptr");
	pBoolPtrType=pLSInterface->FindLSType("boolptr");
	pFloatPtrType=pLSInterface->FindLSType("floatptr");
	pBytePtrType=pLSInterface->FindLSType("byteptr");

	RegisterCommands();
	RegisterAliases();
	RegisterDataTypes();
	RegisterTopLevelObjects();

	return true;
}

// shutdown sequence
void LSMScheduler::Shutdown()
{
	UnRegisterTopLevelObjects();
	UnRegisterDataTypes();
	UnRegisterAliases();
	UnRegisterCommands();
	while(pEvents)
	{
		CScheduledEvent *pEvent=pEvents;
		pEvent->Remove();
		delete pEvent;
	}
}


int CMD_Scheduler(int argc, char *argv[]);

void LSMScheduler::RegisterCommands()
{
	// add any commands
	pLSInterface->AddCommand("Scheduler",CMD_Scheduler,true,false);
}

void LSMScheduler::RegisterAliases()
{
	// add any aliases
}

void LSMScheduler::RegisterDataTypes()
{
	// add any datatypes
	// pMyType = new MyType;
	// pLSInterface->AddLSType(*pMyType);

#define DATATYPE(_class_,_variable_) _variable_ = new _class_; pLSInterface->AddLSType(*_variable_);
#include "DataTypes.h"
#undef DATATYPE

}

bool TLO_ScheduledEvent(int argc, char *argv[], LSTYPEVAR &Dest);

void LSMScheduler::RegisterTopLevelObjects()
{
	// add any Top-Level Objects
	pLSInterface->AddTopLevelObject("ScheduledEvent",TLO_ScheduledEvent);
}

void LSMScheduler::UnRegisterCommands()
{
	// remove commands
	pLSInterface->RemoveCommand("Scheduler");
}
void LSMScheduler::UnRegisterAliases()
{
	// remove aliases
}
void LSMScheduler::UnRegisterDataTypes()
{
	// remove data types
	//pLSInterface->RemoveLSType(*pMyType);
	//delete pMyType;

#define DATATYPE(_class_,_variable_) pLSInterface->RemoveLSType(*_variable_); delete _variable_;
#include "DataTypes.h"
#undef DATATYPE

}
void LSMScheduler::UnRegisterTopLevelObjects()
{
	// remove Top-Level Objects
	pLSInterface->RemoveTopLevelObject("ScheduledEvent");
}

void LSMScheduler::Pulse()
{
	// Anything to be processed once per "frame".  Frames are not at precise intervals, and
	// are product-dependent.  For example, Inner Space frames are dependent on the framerate
	// of the game.

	for (unsigned long N = 0 ; N < 5 ; N++)
	{
		if (!pEvents)
			return;
		if (!pEvents->Ready())
			return;
		pEvents->Execute();
	}

}

void LSMScheduler::AddPreprocessorDefinitions()
{
	pLSInterface->AddPreprocessorDefinition("LSMSCHEDULER","1.00");
}

bool TLO_ScheduledEvent(int argc, char *argv[], LSTYPEVAR &Dest)
{
	if (!argc)
		return false;
	char Temp[2048];
	strcpy(Temp,argv[0]);
	strlwr(Temp);
	if (Dest.Ptr=EventMap[Temp])
	{
		Dest.Type=pScheduledEventType;
		return true;
	}
	return false;
}

int CMD_Scheduler(int argc, char *argv[])
{
	// command in|at <time> every <seconds>
	if (argc<2)
	{
		printf("Syntax: %s -list|-clear|-remove <name>|-add <name> <command> [in|at <time>] [every <interval>]",argv[0]);
		return 0;
	}

	if (!stricmp(argv[1],"-list"))
	{
		printf("Scheduled Events\n----------------------");
		CScheduledEvent *pEvent=pEvents;
		while(pEvent)
		{
			if (unsigned long Recurrence=pEvent->GetSecondsRecurring())
			{
				printf("[%s] every %s, next at %s. '%s'",pEvent->GetName(),CScheduledEvent::SecondsToIntervalString(Recurrence),CScheduledEvent::TimeToTimeString(pEvent->GetNextRun()),pEvent->GetCommand());
			}
			else
			{
				printf("[%s] at %s. '%s'",pEvent->GetName(),CScheduledEvent::TimeToTimeString(pEvent->GetNextRun()),pEvent->GetCommand());
			}
			pEvent=pEvent->GetNext();
		}
	}
	else if (!stricmp(argv[1],"-remove"))
	{
		if (argc<3)
		{
			printf("Usage: %s -remove <name>",argv[0]);
			return 0;
		}
		char Temp[512];
		strcpy(Temp,argv[2]);
		strlwr(Temp);
		if (CScheduledEvent *pEvent=EventMap[Temp])
		{
			pEvent->Remove();
			delete pEvent;
		}
	}
	else if (!stricmp(argv[1],"-clear"))
	{
		while(pEvents)
		{
			CScheduledEvent *pEvent=pEvents;
			pEvent->Remove();
			delete pEvent;
		}
		printf("Scheduler: Events cleared");
	}
	else if (!stricmp(argv[1],"-add"))
	{
		if (argc<5)
		{
			printf("Usage: %s -add <name> <command> [in|at <time>] [every <interval>]",argv[0]);
			return 0;
		}
		// name=2
		char Temp[512];
		strcpy(Temp,argv[2]);
		strlwr(Temp);
		if (CScheduledEvent *pEvent=EventMap[Temp])
		{
			pEvent->Remove();
			delete pEvent;
		}

		// command=3
		int arg=4;
		time_t Next=0;
		unsigned long Recurrence=0;
		if (!stricmp(argv[arg],"in"))
		{
			arg++;
			if (argc<=arg)
			{
				printf("Usage: %s -add <name> <command> [in|at <time>] [every <interval>]",argv[0]);
				return 0;
			}
			Next=CScheduledEvent::IntervalStringToSeconds(argv[arg]);
		}
		else if (!stricmp(argv[arg],"at"))
		{
			arg++;
			if (argc<=arg)
			{
				printf("Usage: %s -add <name> <command> [in|at <time>] [every <interval>]",argv[0]);
				return 0;
			}
			printf("Scheduler: 'at' parsing is currently incomplete.  Please use another method for now.");
			return 0;
			Next=CScheduledEvent::TimeStringToTime(argv[arg]);
		}
		if (!stricmp(argv[arg],"every"))
		{
			arg++;
			if (argc<=arg)
			{
				printf("Usage: %s -add <name> <command> [in|at <time>] [every <interval>]",argv[0]);
				return 0;
			}
			Recurrence=CScheduledEvent::IntervalStringToSeconds(argv[arg]);
			if (!Next)
			{
				Next=time(0)+Recurrence;
			}
		}
		CScheduledEvent *pEvent=new CScheduledEvent(argv[2],argv[3],Next,Recurrence);
		pEvent->Insert();
		return 0;
	}
	else
	{
		printf("Syntax: %s -list|-clear|-remove <name>|-add <name> <command> [in|at <time>] [every <interval>]",argv[0]);
	}

	return 0;
}


