#include "LSMScheduler.h"
#include "scheduledevent.h"

CScheduledEvent *pEvents=0;
map<string,CScheduledEvent *> EventMap;

CScheduledEvent::CScheduledEvent(const char *p_Name,const char *p_Command, unsigned long p_NextRun, unsigned long p_SecondsRecurring)
{
	m_Command=strdup(p_Command);
	m_Name=strlwr(strdup(p_Name));
	NextRun=p_NextRun;
	SecondsRecurring=p_SecondsRecurring;
	bInserted=false;
	pNext=0;
	pPrev=0;
	EventMap[m_Name]=this;
}

CScheduledEvent::~CScheduledEvent(void)
{
	EventMap[m_Name]=0;
	free(m_Name);
	free(m_Command);
}

void CScheduledEvent::Insert()
{
	if (bInserted)
		return;
	bInserted=true;
	if (!pEvents)
	{
		pEvents=this;
		return;
	}
	CScheduledEvent *pLast=0;
	CScheduledEvent *pCurrent=pEvents;
	while(pCurrent)
	{
		if (pCurrent->NextRun>NextRun)
		{
			pCurrent->pPrev=this;
			if (pLast)
				pLast->pNext=this;
			else
			{
				pEvents=this;
				pPrev=pLast;
			}
			pNext=pCurrent;
			return;
		}

		pLast=pCurrent;
		pCurrent=pCurrent->pNext;
	}
	
	pLast->pNext=this;
	pPrev=pLast;
}

void CScheduledEvent::Remove()
{
	if (!bInserted)
		return;
	bInserted=false;
	if (pPrev)
		pPrev->pNext=pNext;
	else
		pEvents=pNext;
	if (pNext)
		pNext->pPrev=pPrev;
}

void CScheduledEvent::Execute()
{
	Remove();
	pLSInterface->ExecuteCommand(m_Command);
	if (!SecondsRecurring)
	{
		delete this;
		return;
	}
	NextRun+=SecondsRecurring;
	Insert();
}

bool CScheduledEvent::Ready()
{
	return ((unsigned long)time(0)>=NextRun);
}

unsigned long CScheduledEvent::IntervalStringToSeconds(char *Interval)
{
	char Temp[256];
	strncpy(Temp,Interval,256);
	Temp[255]=0;
	unsigned long Total=0;
	if (char *pColon=strchr(Temp,':'))
	{
		unsigned long Values[5];
		int nValue=0;
		char *pToken=strtok(Temp,":");
		do
		{
			Values[nValue]=atoi(pColon);
			nValue++;
			if (nValue>=5)
				break;
		}
		while (pToken=strtok(0,":"));
		
		// seconds
		nValue--;
		Total=Values[nValue];
		if (nValue>=0)
		{
			// minutes
			nValue--;
			Total+=Values[nValue]*60;
			if (nValue>=0)
			{
				// hours
				nValue--;
				Total+=Values[nValue]*60*60;
				if (nValue>=0)
				{
					// days
					nValue--;
					Total+=Values[nValue]*60*60*24;
					if (nValue>=0)
					{
						// years
						nValue--;
						Total+=Values[nValue]*60*60*24*365;
					}
				}
			}
		}

		return Total;
	}
	strlwr(Temp);

	static map<string,float> UnitMap;
	static bool UnitMapInitialized=false;

	float Number=1;

	if (!UnitMapInitialized)
	{
		UnitMap["second"]=UnitMap["s"]=1;
		UnitMap["minute"]=UnitMap["m"]=60;
		UnitMap["hour"]=UnitMap["h"]=60*60;
		UnitMap["day"]=UnitMap["d"]=60*60*24;
		UnitMap["year"]=UnitMap["y"]=60*60*24*365;
		UnitMapInitialized=true;
	}

	char *pIn=Temp;

	char *pBegin=Temp;
	unsigned long Type=0;

	do
	{
		switch(char c=*pIn)
		{
		case '.':
			if (Type==1)
				break;
		case ',':
		case ' ':
		case '\t':
			// split
			*pIn=0;
		case 0:
			if (Type==1)
			{
				Number=(float)atof(pBegin);
			}
			else if (Type==2)
			{
				if (pIn-pBegin>1 && pIn[-1]=='s')
				{
					pIn[-1]=0;
				}
				float Unit=UnitMap[pBegin];
				Total+=(unsigned long)(Unit*Number);
				Number=0;
			}
			Type=0;
			pBegin=pIn;
			if (c==0)
				return Total;
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			// number
			if (Type!=1)
			{
				if (Type==2)
				{
					char cTemp=*pIn;
					*pIn=0;
					if (pIn-pBegin>1 && pIn[-1]=='s')
					{
						pIn[-1]=0;
					}
					float Unit=UnitMap[pBegin];
					Total+=(unsigned long)(Unit*Number);
					Number=0;
					*pIn=cTemp;
				}
				pBegin=pIn;
				Type=1;
			}
			break;
		default:
			// text
			if (Type!=2)
			{
				if (Type==1)
				{
					char cTemp=*pIn;
					*pIn=0;
					Number=(float)atof(pBegin);
					*pIn=cTemp;
				}
				pBegin=pIn;
				Type=2;
			}
			break;
		}
		pIn++;
	}
	while(1);

	return 0;
}
char *CScheduledEvent::SecondsToIntervalString(unsigned long TotalSeconds)
{
	static char Buffer[256];
	char *pOut=Buffer;
	if (TotalSeconds>60*60*24*365)
	{
		// y:ddd:hh:mm:ss
		unsigned long Seconds=TotalSeconds%60;
		unsigned long Minutes=(TotalSeconds/60)%60;
		unsigned long Hours=(TotalSeconds/60/60)%24;
		unsigned long Days=(TotalSeconds/60/60/24)%365;
		unsigned long Years=TotalSeconds/60/60/24/365;
		sprintf(Buffer,"%d:%03d:%02d:%02d:%02d",Years,Days,Hours,Minutes,Seconds);
	}
	else if (TotalSeconds>60*60*24)
	{
		// d:hh:mm:ss
		unsigned long Seconds=TotalSeconds%60;
		unsigned long Minutes=(TotalSeconds/60)%60;
		unsigned long Hours=(TotalSeconds/60/60)%24;
		unsigned long Days=TotalSeconds/60/60/24;
		sprintf(Buffer,"%d:%02d:%02d:%02d",Days,Hours,Minutes,Seconds);
	}
	else if (TotalSeconds>60*60)
	{
		// h:mm:ss
		unsigned long Seconds=TotalSeconds%60;
		unsigned long Minutes=(TotalSeconds/60)%60;
		unsigned long Hours=TotalSeconds/60/60;
		sprintf(Buffer,"%d:%02d:%02d",Hours,Minutes,Seconds);
	}
	else 
	{
		// minutes:seconds
		unsigned long Seconds=TotalSeconds%60;
		unsigned long Minutes=TotalSeconds/60;
		sprintf(Buffer,"%d:%02d",Minutes,Seconds);
	}
	return Buffer;
}
time_t CScheduledEvent::TimeStringToTime(char *Time)
{
	return 0;
}
char *CScheduledEvent::TimeToTimeString(time_t Time)
{
	char static Buffer[256];
	time_t now=time(0);
	struct tm Tomorrow=*localtime(&now);
	Tomorrow.tm_mday++;
	Tomorrow.tm_hour=0;
	Tomorrow.tm_min=0;
	Tomorrow.tm_sec=0;

	time_t TimeTomorrow=mktime(&Tomorrow);
	struct tm* pTime=localtime(&Time);

	if (Time>TimeTomorrow)
	{
		if (pTime->tm_hour>=12)
		{
			sprintf(Buffer,"%04d-%02d-%02d %02d:%02d:%02dpm",pTime->tm_year+1900,pTime->tm_mon+1,pTime->tm_mday,pTime->tm_hour-11,pTime->tm_min,pTime->tm_sec);
		}
		else
		{
			if (!pTime->tm_hour)
				pTime->tm_hour=12;
			sprintf(Buffer,"%04d-%02d-%02d %02d:%02d:%02dam",pTime->tm_year+1900,pTime->tm_mon+1,pTime->tm_mday,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
		}
	}
	else
	{
		if (pTime->tm_hour>=12)
		{
			sprintf(Buffer,"%02d:%02d:%02dpm",pTime->tm_hour-11,pTime->tm_min,pTime->tm_sec);
		}
		else
		{
			if (!pTime->tm_hour)
				pTime->tm_hour=12;
			sprintf(Buffer,"%02d:%02d:%02dam",pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
		}
	}
	return Buffer;
}



bool LSScheduledEventType::GetMember(LSVARPTR VarPtr, PCHAR Member, int argc, char *argv[], LSTYPEVAR &Dest)
{
#define pEvent ((CScheduledEvent *)VarPtr.Ptr)
	if (!VarPtr.Ptr)
		return false;
	PLSTYPEMEMBER pMember=LSScheduledEventType::FindMember(Member);
	if (!pMember)
		return false;
	switch((LSScheduledEventMembers)pMember->ID)
	{
	case Name:
		Dest.Ptr=pEvent->GetName();
		Dest.Type=pStringType;
		return true;
	case Command:
		Dest.Ptr=pEvent->GetCommand();
		Dest.Type=pStringType;
		return true;
	case Recurrence:
		Dest.DWord=pEvent->GetSecondsRecurring();
		Dest.Type=pIntType;
		return true;
	case TimeUntil:
		Dest.DWord=pEvent->GetNextRun()-time(0);
		Dest.Type=pIntType;
		return true;
	case Time:
		{
			time_t CurTime=(time_t)pEvent->GetNextRun();
			Dest.Ptr= localtime( &CurTime ); 	
			Dest.Type=pTimeType;
		}
		return true;
	case Next:
		if (Dest.Ptr=pEvent->GetNext())
		{
			Dest.Type=pScheduledEventType;
			return true;
		}
		return false;
	case Previous:
		if (Dest.Ptr=pEvent->GetPrev())
		{
			Dest.Type=pScheduledEventType;
			return true;
		}
		return false;
	}
	return false;
#undef pEvent
}

bool LSScheduledEventType::GetMethod(LSVARPTR &VarPtr, char *Method, int argc, char *argv[])
{
#define pEvent ((CScheduledEvent *)VarPtr.Ptr)
	if (!VarPtr.Ptr)
		return false;
	PLSTYPEMEMBER pMethod=LSScheduledEventType::FindMethod(Method);
	if (!pMethod)
		return false;
	switch((LSScheduledEventMethods)pMethod->ID)
	{
	case Remove:
		pEvent->Remove();
		delete pEvent;
		VarPtr.Ptr=0;
		return true;
	case Reset:
		{
			unsigned long Add;
			if (argc)
				Add=atoi(argv[0]);
			else
				Add=pEvent->GetSecondsRecurring();
			pEvent->Remove();
			pEvent->SetNextRun(time(0)+Add);
			pEvent->Insert();
		}
		return true;
	case SetCommand:
		if (argc)
		{
			pEvent->SetCommand(argv[0]);
			return true;
		}
		return false;
	case SetRecurrence:
		if (argc)
		{
			pEvent->SetSecondsRecurring(atoi(argv[0]));
			return true;
		}
		return false;
	case Execute:
		pLSInterface->ExecuteCommand(pEvent->GetCommand());
		return true;
	}
	return false;
#undef pEvent
}



