#pragma once

class CScheduledEvent
{
public:
	CScheduledEvent(const char *p_Name, const char *p_Command, unsigned long p_NextRun, unsigned long p_SecondsRecurring);
	~CScheduledEvent(void);

	bool Ready();
	void Insert();
	void Remove();
	void Execute();

	static unsigned long IntervalStringToSeconds(char *Interval);
	static char *SecondsToIntervalString(unsigned long Seconds);
	static time_t TimeStringToTime(char *Time);
	static char *TimeToTimeString(time_t Time);


	inline char *GetName() {return m_Name;}
	inline char *GetCommand() {return m_Command;}
	inline unsigned long GetNextRun() {return NextRun;}
	inline unsigned long GetSecondsRecurring() {return SecondsRecurring;}

	inline void SetNextRun(unsigned long p_NextRun) {NextRun=p_NextRun;}
	inline void SetSecondsRecurring(unsigned long p_SecondsRecurring) {SecondsRecurring=p_SecondsRecurring;}
	inline void SetCommand(char *p_Command) {free(m_Command);m_Command=strdup(p_Command);}

	inline CScheduledEvent *GetNext() {return pNext;}
	inline CScheduledEvent *GetPrev() {return pPrev;}
protected:
	unsigned long NextRun; // unix timestamp
	unsigned long SecondsRecurring;
	char *m_Command;
	char *m_Name;

	bool bInserted;

	CScheduledEvent *pNext;
	CScheduledEvent *pPrev;
};

class LSScheduledEventType : public LSType
{
public:
	// Set up enumeration with all of the possible members of this type
	static enum LSScheduledEventMembers
	{
		Name=1,
		Command=2,
		Recurrence=3,
		TimeUntil=4,
		Time=5,
		Next=6,
		Previous=7,
	};
	static enum LSScheduledEventMethods
	{
		Remove=1,
		Reset=2,
		SetCommand=3,
		SetRecurrence=4,
		Execute=5,
	};

	LSScheduledEventType():LSType("scheduledevent")
	{
		// Add each member name to the list, the TypeMember macro automatically uses
		// the name used in the enumeration

		TypeMember(Name);
		TypeMember(Command);
		TypeMember(Recurrence);
		TypeMember(TimeUntil);
		TypeMember(Time);
		TypeMember(Next);
		TypeMethod(Previous);

		TypeMethod(Remove);
		TypeMethod(Reset);
		TypeMethod(SetCommand);
		TypeMethod(SetRecurrence);
		TypeMethod(Execute);
	};

	~LSScheduledEventType() {}

	bool GetMember(LSVARPTR VarPtr, PCHAR Member, int argc, char *argv[], LSTYPEVAR &Dest);
	bool GetMethod(LSVARPTR &VarPtr, char *Method, int argc, char *argv[]);

	bool ToString(LSVARPTR VarPtr, PCHAR Destination) 
	{
		if (!VarPtr.Ptr)
			return false;
		strcpy(Destination,((CScheduledEvent *)VarPtr.Ptr)->GetName());
		return true;
	}

	// The following member functions should be modified if you wish to
	// support scripting variables

	void InitVariable(LSVARPTR &VarPtr) {}
	void FreeVariable(LSVARPTR &VarPtr) {}

	bool FromData(LSVARPTR &VarPtr, LSTYPEVAR &Source)
	{
		return false;
	}

	bool FromString(LSVARPTR &VarPtr, PCHAR Source)
	{
		return false;
	}
};
